/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.containers;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.ListIterator;

import net.sourceforge.mysources.thinkinjava4.utils.Countries;
import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * Things you can do with Lists.
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2013-1-1
 * @since 1.1.2
 */
public class Lists {
	private static boolean b;
	private static String s;
	private static int i;
	private static Iterator<String> it;
	private static ListIterator<String> lit;
	
	public static void basicTest(List<String> a){
		a.add(1, "x"); // Add at location 1
		a.add("x"); // add at end
		// Add a collection;
		a.addAll(Countries.names(25));
		// Add a collection starting at location 3
		a.addAll(3, Countries.names(25));
		b = a.contains("1"); // Is it in there
		// Is the entire collection in there
		b = a.containsAll(Countries.names(25));
		// Lists allow random access, whick is cheap
		// for ArrayList, expensive for LinkedList
		s = a.get(1); // Get (typed object at location
		i = a.indexOf("1"); // Tell index of object
		b = a.isEmpty(); // Any elements inside
		it = a.iterator(); // Ordinary Iterator
		lit = a.listIterator(); // ListIterator
		lit = a.listIterator(3); // Remove location
		i = a.lastIndexOf("1"); // Last match
		a.remove(1); // Remove location
		a.remove("3"); // Remove this object
		a.set(1, "y"); // Set location 1 to "y"
		// Keep everything that's in the argument
		// (the intersection of the two sets)
		a.retainAll(Countries.names(25));
		i = a.size(); // How big is it
		a.clear(); // Remove all elements
	}
	
	public static void iterMotion(List<String> a){
		ListIterator<String> it = a.listIterator();
		b = it.hasNext();
		b = it.hasPrevious();
		s = it.next();
		i = it.nextIndex();
		s = it.previous();
		i = it.previousIndex();
	}
	
	public static void iterManipulation(List<String> a){
		ListIterator<String> it = a.listIterator();
		it.add("47");
		// Must move to an element after add()
		it.next();
		// Remove the element after the newly produced one
		it.remove();
		// Must move to an element after remove
		it.next();
		// Change the element after the deleted one
		it.set("47");
	}
	
	public static void testVisual(List<String> a){
		Print.print(a);
		List<String> b = Countries.names(25);
		Print.print("b = " + b);
		a.addAll(b);
		a.addAll(b);
		Print.print(a);
		// Insert, remove , and replace elements
		// using a ListIterator
		ListIterator<String> x = a.listIterator(a.size()/2);
		x.add("one");
		Print.print(a);
		Print.print(x.next());
		x.remove();
		Print.print(x.next());
		x.set("47");
		Print.print(a);
		// Traverse the list backwards
		x = a.listIterator(a.size());
		while(x.hasPrevious())
			Print.printnb(x.previous() + " ");
		Print.print();
		Print.print("testVisual finished");
	} 
	
	// There are some things that only LinkedLists can do
	public static void testLinkedList(){
		LinkedList<String> ll = new LinkedList<String>();
		ll.addAll(Countries.names(25));
		Print.print(ll);
		// Treat it like a stack, pushing
		ll.addFirst("one");
		ll.addFirst("two");
		Print.print(ll);
		
		// Like "peeking" at the top of a stack
		Print.print(ll.getFirst());
		// Like poping a stack
		Print.print(ll.removeFirst());
		Print.print(ll.removeFirst());
		
		// Treat it like a queue, pulling elements
		// off the tail end;
		Print.print(ll.removeLast());
		Print.print(ll);
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// Make and fill a new list each time
		basicTest(
				new LinkedList<String>(Countries.names(25)));
		basicTest(
				new ArrayList<String>(Countries.names(25)));
		
		iterMotion(
				new LinkedList<String>(Countries.names(25)));
		iterMotion(
				new ArrayList<String>(Countries.names(25)));
		
		iterManipulation(
				new LinkedList<String>(Countries.names(25)));
		iterManipulation(
				new ArrayList<String>(Countries.names(25)));		

		testLinkedList();
	}

}


