/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.containers;

import java.util.AbstractMap;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import net.sourceforge.mysources.thinkinjava4.utils.Countries;
import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * A Map implemented with ArrayLists
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2013-2-27
 * @since 1.1.2
 */
public class SlowMap<K, V> extends AbstractMap<K, V> {
	private List<K> keys = new ArrayList<K>();
	private List<V> values = new ArrayList<V>();
	
	@Override
	public V get(Object key) { // key is type Object, not K
		if(!keys.contains(key))
			return null;
		return values.get(keys.indexOf(key));
	}

	@Override
	public V put(K key, V value) {
		V oldValue = get(key); // The old value or null
		if(!keys.contains(key)){
			keys.add(key);
			values.add(value);
		} else
			values.set(keys.indexOf(key), value);
		return oldValue;
	}

	@Override
	public Set<java.util.Map.Entry<K, V>> entrySet() {
		Set<Map.Entry<K, V>> set = new HashSet<Map.Entry<K, V>>();
		Iterator<K> ki = keys.iterator();
		Iterator<V> vi = values.iterator();
		while(ki.hasNext())
			set.add(new MapEntry<K,V>(ki.next(), vi.next()));
		return set;
	}

	public static void main(String[] args){
		SlowMap<String,String> m = new SlowMap<String,String>();
		m.putAll(Countries.capitals(15));
		Print.print(m);
		Print.print(m.get("BULGARIA"));
		Print.print(m.entrySet());
	}
}


