/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.containers;

import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 使用特定的Set实现类型而必须定义的方法
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2013-2-20
 * @since 1.1.2
 */
class SetType{
	int i;
	public SetType(int n){ i = n; }
	@Override
	public boolean equals(Object o) {
		return o instanceof SetType && (i == ((SetType)o).i);
	}
	@Override
	public String toString() {
		return Integer.toString(i);
	}	
}

class HashType extends SetType{

	public HashType(int n) {
		super(n);
	}

	@Override
	public int hashCode() {
		return i;
	}	
}

class TreeType extends SetType implements Comparable<TreeType>{

	public TreeType(int n) {
		super(n);
	}

	@Override
	public int compareTo(TreeType arg) {
		return (arg.i < i ? -1 : (arg.i == i ? 0 : 1));
	}	
}

public class TypesForSets {
	static <T> Set<T> fill(Set<T> set, Class<T> type){
		try{
			for(int i = 0; i < 10; i++)
				set.add(type.getConstructor(int.class).newInstance(i));
		}catch(Exception e){
			throw new RuntimeException(e);
		}
		
		return set;
	}
	
	static <T> void test(Set<T> set, Class<T> type){
		fill(set, type);
		fill(set, type); // Try to add duplicates
		fill(set, type);
		Print.print(set);
	}
	
	public static void main(String[] args){
		test(new HashSet<HashType>(), HashType.class);
		test(new LinkedHashSet<HashType>(), HashType.class);
		test(new TreeSet<TreeType>(), TreeType.class);
		
		// Things that don't work
		test(new HashSet<SetType>(), SetType.class);
		test(new HashSet<TreeType>(), TreeType.class);
		test(new LinkedHashSet<SetType>(), SetType.class);
		test(new LinkedHashSet<TreeType>(), TreeType.class);
		
		test(new TreeSet<SetType>(), SetType.class);
		test(new TreeSet<HashType>(), HashType.class);
	}
}


