/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.mysources.thinkinjava4.enumerated;

import java.util.EnumSet;

import net.sourceforge.mysources.thinkinjava4.utils.Print;
/**
 * 与使用匿名内部类相比较，定义常量相关方法的语法更
 * 高效，简洁
 *
 * @author Lucky Yang
 * @since 0.1
 */
public class CarWash {
	public enum Cycle{
		UNDERBODY{
			void action(){
				Print.print("Spraying the underbody");
			}
		},
		WHEELWASH{
			void action(){
				Print.print("Washing the wheels");
			}			
		},
		PREWASH{
			void action(){
				Print.print("Loosening the dirt");
			}			
		},
		BASIC{
			void action(){
				Print.print("The basic wash");
			}			
		},
		HOTWAX{
			void action(){
				Print.print("Applying hot wax");
			}			
		},
		RINSE{
			void action(){
				Print.print("Rinsing");
			}			
		},
		BLOWDRY{
			void action(){
				Print.print("Blowing dry");
			}
		};
		
		abstract void action();
	}
	
	// 固定服务
	EnumSet<Cycle> cycles = EnumSet.of(Cycle.BASIC, Cycle.RINSE);
	
	public void add(Cycle cycle){
		cycles.add(cycle);
	}
	
	public void washCar(){
		for(Cycle c : cycles){
			c.action();
		}
	}
	
	public String toString(){
		return cycles.toString();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		CarWash wash = new CarWash();
		Print.print(wash);
		wash.washCar();
		
		// order of addition is unimportant
		wash.add(Cycle.BLOWDRY);
		wash.add(Cycle.BLOWDRY); // Duplicates ignored
		wash.add(Cycle.RINSE);
		wash.add(Cycle.HOTWAX);
		Print.print(wash);
		wash.washCar();
	}
}
