/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.enumerated;

import static net.sourceforge.mysources.thinkinjava4.enumerated.AlarmPoints.BATHROOM;
import static net.sourceforge.mysources.thinkinjava4.enumerated.AlarmPoints.KITCHEN;
import static net.sourceforge.mysources.thinkinjava4.enumerated.AlarmPoints.LOBBY;

import java.util.EnumMap;
import java.util.Map;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 演示如何使用EnumMap。使用了命令行模式，命令行模式首先需要
 * 一个只有单一方法的接口，然后从该接口实现具有各种不同行为的
 * 多个子类。
 *
 * @author Lucky Yang
 * @since 0.1
 */
public class EnumMaps {
	interface Command {void action();}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		EnumMap<AlarmPoints, Command> em = 
			new EnumMap<AlarmPoints, Command>(AlarmPoints.class);
		
		em.put(KITCHEN, new Command(){
			public void action(){
				Print.print("Kitchen fire");
			}
		});
		
		em.put(BATHROOM, new Command(){
			public void action(){
				Print.print("Bathroom fire");
			}
		});	
		
		// Map循环
		for(Map.Entry<AlarmPoints, Command> e : em.entrySet()){
			Print.printnb(e.getKey() + ":");
			e.getValue().action();
		}
		
		// If there's no value for a particular key
		try{
			em.get(LOBBY).action();
		}catch(Exception e){
			Print.print(e);
		}
	}
}
