/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.mysources.thinkinjava4.enumerated;

import java.util.Iterator;

import net.sourceforge.mysources.thinkinjava4.utils.Generator;
import net.sourceforge.mysources.thinkinjava4.utils.Print;
import net.sourceforge.mysources.thinkinjava4.utils.TextFile;
/**
 * 自动售货机
 *
 * 枚举类型非常适合用来创建状态机，一个状态机可以具有有限
 * 个特定的状态，它通常根据输入，从一个状态转移到下一个状态，
 * 不过也可能存在瞬时状态(transient states),而一旦任务
 * 结束，状态机会立刻离开瞬时状态。
 * 
 * @author Lucky Yang
 * @since 0.1
 */
public class VendingMachine {
	private static State state = State.RESTING;
	private static int amount = 0;
	private static Input selection = null;
	enum StateDuration {TRANSIENT } // Tagging enum
	enum State {
		RESTING{
			void next(Input input){
				switch(Category.categorize(input)){
				case MONEY:
					amount += input.amount();
					state = ADDING_MONEY;
					break;
				case SHUT_DOWN:
					state = TERMINAL;
					break;
				default:
				}
			}
		},
		ADDING_MONEY{
			void next(Input input){
				switch(Category.categorize(input)){
				case MONEY:
					amount += input.amount();
					break;
				case ITEM_SELECTION:
					selection = input;
					if(amount < selection.amount())
						Print.print("Insufficent money for " + selection);
					else
						state = DISPENSING;
					break;
				case QUIT_TRANSACTION:
					state = GIVING_CHANGE;
					break;
				case SHUT_DOWN:
					state = TERMINAL;
					break;
				default:
				}
			}			
		},
		DISPENSING(StateDuration.TRANSIENT){
			void next(){
				Print.print("here is your " + selection);
				amount -= selection.amount();
				state = GIVING_CHANGE;
			}			
		},
		GIVING_CHANGE(StateDuration.TRANSIENT){
			void next(){
				if (amount > 0){
					Print.print("Your change :" + amount);
					amount = 0;
				}
				state = RESTING;
			}			
		},
		TERMINAL{
			void output(){ Print.print("Halted");}
		};
		private boolean isTrannsient = false;
		State(){};
		State(StateDuration trans){ this.isTrannsient = true;};
		void next(Input input){
			throw new RuntimeException("Only call " +
					"next(Input input) for non-transient states");
		}
		void next(){
			throw new RuntimeException("Only call " +
					"next( for StateDuration.TRANSIENT states");
		}
		void output(){Print.print(amount);}
	}
	static void run(Generator<Input> gen){
		while(state != State.TERMINAL){
			state.next(gen.next());
			while(state.isTrannsient)
				state.next();
			
			state.output();
		}
	}
	public static void main(String[] args){
		Generator<Input> gen = new RandomInputGenerator();
		if (args.length == 1)
			gen = new FileInputGenerator(args[0]);
		run(gen);
	}
}

// For a base sanity check
class RandomInputGenerator implements Generator<Input>{
	public Input next(){ return Input.randomSelect();}
}

// Create Inputs from a file of ';' - separated strings:
class FileInputGenerator implements Generator<Input>{
	private Iterator<String> input;
	public FileInputGenerator(String fileName){
		input = new TextFile(fileName, ";").iterator();
	}
	public Input next(){
		if (! input.hasNext())
			return null;
		
		return Enum.valueOf(Input.class, input.next().trim());
	}
}