/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.io;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.ByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.charset.Charset;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 数据转换
 * <p>
 * Converting text to and from ByteBuffers 
 *
 * @anthor Lucky Yang
 * @Date 2012-11-24
 * @since 1.0.0
 */
public class BufferToText {
	private static final int BSIZE = 1024;
	
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		FileChannel fc = new FileOutputStream("data2.txt").getChannel();
		fc.write(ByteBuffer.wrap("Some text".getBytes()));
		fc.close();
		
		fc = new FileInputStream("data2.txt").getChannel();
		ByteBuffer buff = ByteBuffer.allocate(BSIZE);
		fc.read(buff);
		buff.flip();
		// Doesn't work
		Print.print(buff.asCharBuffer());
		// Decode using this system's default Charset
		buff.rewind();
		String encoding = System.getProperty("file.encoding");
		Print.print("Decoded using " + encoding + ":" +
				" " + Charset.forName(encoding).decode(buff));
		
		// Or, we could encode with something that will print
		fc = new FileOutputStream("data2.txt").getChannel();
		fc.write(ByteBuffer.wrap(
				"Some text".getBytes("UTF-16BE")));
		fc.close();
		// Now try reading again
		fc = new FileInputStream("data2.txt").getChannel();
		buff.clear();
		fc.read(buff);
		buff.flip();
		Print.print(buff.asCharBuffer());
		
		// Use a CharBuffer to write through
		fc = new FileOutputStream("data2.txt").getChannel();
		buff = ByteBuffer.allocate(24); // More than needed
		buff.asCharBuffer().put("Some text");
		fc.write(buff);
		fc.close();
		// Read and display
		fc = new FileInputStream("data2.txt").getChannel();
		buff.clear();
		fc.read(buff);
		buff.flip();
		Print.print(buff.asCharBuffer());
		
	}

}


