/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.io;

import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.ByteBuffer;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 对映射文件的部分加锁
 * <p>
 * Locking portions of a mapped file.
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-1
 * @since 1.0.0
 */
public class LockingMappedFiles {
	static final int LENGTH = 0x8FFFFFF; // 128 MB
	static FileChannel fc;
	/**
	 * @param args
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IOException {
		fc = new RandomAccessFile("test.dat", "rw").getChannel();
		MappedByteBuffer out = fc.map(FileChannel.MapMode.READ_WRITE, 0, LENGTH);
		for(int i = 0; i < LENGTH; i++)
			out.put((byte)'x');
		new LockAndModify(out, 0, 0 + LENGTH/3);
		new LockAndModify(out, LENGTH/2, LENGTH/2 + LENGTH/4);
	}
	private static class LockAndModify extends Thread{

		private ByteBuffer buff;
		private int start, end;
		public LockAndModify(ByteBuffer mbb, int start, int end) {
			this.start = start;
			this.end = end;
			mbb.limit(end);
			mbb.position(start);
			buff = mbb.slice();
			start();
		}	
		public void run(){
			try{
				// Exclusive lock with no overlap
				FileLock fl = fc.lock(start, end, false);
				Print.print("Locked: " + start + " to " + end);
				// Perform modification;
				while(buff.position() < buff.limit() - 1)
					buff.put((byte)(buff.get() + 1));
				fl.release();
				Print.print("Released: " + start + " to " + end);
			}catch(IOException e){
				throw new RuntimeException(e);
			}
		}
	}
}


