/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.io;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.IntBuffer;
import java.nio.channels.FileChannel;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 性能
 * <p>
 *
 * @anthor Lucky Yang
 * @Date 2012-11-25
 * @since 1.0.0
 */
public class MappedIO {
	private static int numOfInts = 4000000;
	private static int numOfUbuffInts = 200000;
	
	private abstract static class Tester{
		private String name;
		public Tester(String name){this.name = name;}
		public void runTest(){
			Print.printnb(name + ": ");
			try{
				long start = System.nanoTime();
				test();
				double duration = System.nanoTime() - start;
				System.out.format("%.2f\n", duration/1.0e9);
			}catch(IOException e){
				throw new RuntimeException(e);
			}
		}
		public abstract void test() throws IOException;
	}
	
	private static Tester[] tests ={
		new Tester("Stream Write"){
			@Override
			public void test() throws IOException {
				DataOutputStream dos = new DataOutputStream(
						new BufferedOutputStream(
								new FileOutputStream(new File("temp.tmp"))));
				for(int i = 0; i < numOfInts; i++)
					dos.writeInt(i);
				dos.close();
			}
		},
		new Tester("Mapped Write"){
			@Override
			public void test() throws IOException {
				FileChannel fc = new RandomAccessFile("temp.tmp", "rw").getChannel();
				IntBuffer ib = fc.map(
						FileChannel.MapMode.READ_WRITE, 0, fc.size()).asIntBuffer();
				for(int i = 0; i < numOfInts; i++)
					ib.put(i);
				fc.close();				
			}
		},	
		new Tester("Stream Read"){
			@Override
			public void test() throws IOException {
				DataInputStream dis = new DataInputStream(
						new BufferedInputStream(
								new FileInputStream(new File("temp.tmp"))));
				for(int i = 0; i < numOfInts; i++)
					dis.readInt();
				dis.close();				
			}
		},
		new Tester("Mapped Read"){
			@Override
			public void test() throws IOException {
				FileChannel fc = new FileInputStream(new File("temp.tmp")).getChannel();
				IntBuffer ib = fc.map(
						FileChannel.MapMode.READ_ONLY, 0, fc.size()).asIntBuffer();
				while(ib.hasRemaining())
					ib.get();
				fc.close();					
			}
		},
		new Tester("Stream Read/Write"){
			@Override
			public void test() throws IOException {
				RandomAccessFile raf = new RandomAccessFile(
						new File("temp.tmp"), "rw");
				raf.writeInt(1);
				for(int i = 0; i < numOfUbuffInts; i++){
					raf.seek(raf.length() - 4);
					raf.writeInt(raf.readInt());
				}
				raf.close();
									
			}
		},
		new Tester("Mapped Read/Write"){
			@Override
			public void test() throws IOException {
				FileChannel fc = new RandomAccessFile(new File("temp.tmp"), "rw").getChannel();
				IntBuffer ib = fc.map(
						FileChannel.MapMode.READ_WRITE, 0, fc.size()).asIntBuffer();
				ib.put(0);
				for(int i = 1; i < numOfUbuffInts; i++){
					ib.put(ib.get(i - 1));
				}				
				fc.close();
			}
		},		
	};
	
	public static void main(String[] args){
		for(Tester test : tests)
			test.runTest();
	}
}


