/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.io;

import java.io.BufferedOutputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Arrays;
import java.util.List;

import net.sourceforge.mysources.thinkinjava4.utils.Print;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Serializer;


/**
 * 使用XOM库读写XML文件
 * <p>
 * Use the XOM library to write and read XML
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-22
 * @since 1.0.0
 */
public class Person {
	private String first, last;
	public Person(String first, String last){
		this.first = first;
		this.last = last;
	}
	// Produce an XML Element from this Person object
	public Element getXML(){
		Element person = new Element("person");
		Element firstName = new Element("first");
		firstName.appendChild(first);
		Element lastName = new Element("last");
		lastName.appendChild(last);
		
		person.appendChild(firstName);
		person.appendChild(lastName);
		return person;
	}
	// Constructor to restore a Person from an XML Element
	public Person(Element person){
		first = person.getFirstChildElement("first").getValue();
		last = person.getFirstChildElement("last").getValue();
	}
	public String toString(){
		return first + " " + last;
	}
	
	// Make it human-readable
	public static void format(OutputStream os, Document doc) throws IOException{
		Serializer serializer = new Serializer(os, "ISO-8859-1");
		serializer.setIndent(4);
		serializer.setMaxLength(60);
		serializer.write(doc);
		serializer.flush();
	}
	
	public static void main(String[] args) throws IOException{
		List<Person> people = Arrays.asList(
				new Person("Dr. Bunsen","Honeydew"),
				new Person("Gonzo","The Great"),
				new Person("Phillip J.","Fry"));
		Print.print(people);
		Element root = new Element("people");
		for(Person p : people)
			root.appendChild(p.getXML());
		Document doc = new Document(root);
		format(System.out, doc);
		format(new BufferedOutputStream(
				new FileOutputStream(
						"People.xml")),doc);
	}
}


