/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.io;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import net.sourceforge.mysources.thinkinjava4.utils.Print;

/**
 * 序列化的应用，序列化大量对象的有效方法
 * <p>
 * Saving the state of a pretend CAD system. 
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-22
 * @since 1.0.0
 */

abstract class Shape implements Serializable{
	public static final int RED = 1, BLUE = 2, GREEN = 3;
	private int xPos, yPos, dimension;
	private static Random rand = new Random(47);
	private static int counter = 0;
	
	public abstract void setColor(int newColor);
	public abstract int getColor();
	public Shape(int xVal, int yVal, int dim){
		xPos = xVal;
		yPos = yVal;
		dimension = dim;
	}
	public String toString(){
		return getClass() + "color[" + getColor() + "] xPos[" +
				xPos + "] yPos[" + yPos + "] dim[" +
						+ dimension + "]\n";
	}
	public static Shape randomFacory(){
		int xVal = rand.nextInt(100);
		int yVal = rand.nextInt(100);
		int dim = rand.nextInt(100);
		switch(counter++ % 3){
			default:
			case 0: return new Circle(xVal, yVal, dim);
			case 1: return new Square(xVal, yVal, dim);
			case 2: return new Line(xVal, yVal, dim);
		}
	}
}

class Circle extends Shape{
	private static int color = RED;
	public Circle(int xVal, int yVal, int dim){
		super(xVal, yVal, dim);
	}
	@Override
	public void setColor(int newColor) {
		color = newColor;
	}
	@Override
	public int getColor() {
		return color;
	}
}

class Square extends Shape{
	private static int color;
	public Square(int xVal, int yVal, int dim){
		super(xVal, yVal, dim);
		color = RED;
	}
	@Override
	public void setColor(int newColor) {
		color = newColor;
	}
	@Override
	public int getColor() {
		return color;
	}
}

class Line extends Shape{
	private static int color = RED;
	public Line(int xVal, int yVal, int dim){
		super(xVal, yVal, dim);
	}
	@Override
	public void setColor(int newColor) {
		color = newColor;
	}
	@Override
	public int getColor() {
		return color;
	}
	
	public static void serializeStaticState(ObjectOutputStream os) throws IOException{
		os.writeInt(color);
	}
	
	public static void deserializeStaticState(ObjectInputStream os) throws IOException{
		color = os.readInt();
	}	
}

public class StoreCADState {

	/**
	 * @param args
	 * @throws IOException 
	 * @throws FileNotFoundException 
	 */
	public static void main(String[] args) throws FileNotFoundException, IOException {
		List<Class<? extends Shape>> shapeTypes =
				new ArrayList<Class<? extends Shape>>();
		
		// Add references to the class objects
		shapeTypes.add(Circle.class);
		shapeTypes.add(Square.class);
		shapeTypes.add(Line.class);
		
		List<Shape> shapes = new ArrayList<Shape>();
		
		// Make some shapes;
		for(int i = 0; i < 10; i++)
			shapes.add(Shape.randomFacory());
		// Set all the static colors to GREEN
		for(int i = 0; i < 10; i++)
			shapes.get(i).setColor(Shape.GREEN);
		// Save the state vector
		ObjectOutputStream out = new ObjectOutputStream(
				new FileOutputStream("CADState.out"));
		out.writeObject(shapeTypes);
		Line.serializeStaticState(out);
		out.writeObject(shapes);
		// Display the shapes
		Print.print(shapes);
	}

}


