/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.utils;

/**
 * Simple generator implementations. 
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-23
 * @since 1.1.2
 */
public class CountingGenerator {
	public static class
	Boolean implements Generator<java.lang.Boolean>{
		private boolean value = false;
		@Override
		public java.lang.Boolean next() {
			value = !value; // Just flips back and forth
			return value;
		}	
	}
	public static class
	Byte implements Generator<java.lang.Byte>{
		private byte value = 0;
		@Override
		public java.lang.Byte next() {
			return value++;
		}	
	}
	static char[] chars = ("abcdefghijklmnopqrstuvwxyz" + 
		    "ABCDEFGHIJKLMNOPQRSTUVWXYZ").toCharArray();
	public static class
	Character implements Generator<java.lang.Character>{
		private int index = -1;
		@Override
		public java.lang.Character next() {
			index = (index + 1) % chars.length;
			return chars[index];
		}	
	}	
	public static class
	String implements Generator<java.lang.String>{
		private int length = 7;
		protected Generator<java.lang.Character> cg = new Character();
		public String(){};
		public String(int length){this.length = length;}
		@Override
		public java.lang.String next() {
			char[] buf = new char[length];
			for(int i = 0; i < length; i++)
				buf[i] = cg.next();
			return new java.lang.String(buf);
		}	
	}
	public static class
	Short implements Generator<java.lang.Short>{
		private short value = 0;
		@Override
		public java.lang.Short next() {
			return value++;
		}	
	}
	public static class
	Integer implements Generator<java.lang.Integer>{
		private int value = 0;
		@Override
		public java.lang.Integer next() {
			return value++;
		}	
	}
	public static class
	Long implements Generator<java.lang.Long>{
		private long value = 0;
		@Override
		public java.lang.Long next() {
			return value++;
		}	
	}
	public static class
	Float implements Generator<java.lang.Float>{
		private float value = 0;
		@Override
		public java.lang.Float next() {
			float result = value;
			value += 1.0;
			return result;
		}	
	}	
	public static class
	Double implements Generator<java.lang.Double>{
		private double value = 0.0;
		@Override
		public java.lang.Double next() {
			double result = value;
			value += 1.0;
			return result;
		}	
	}	
}


