/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.utils;

import java.util.AbstractMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

/**
 * Unlimited-length Map containing sample data.
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-29
 * @since 1.1.2
 */
public class CountingMapData extends AbstractMap<Integer, String> {
	private int size;
	private static String[] chars = 
			"A B C D E F G H I J K L M N O P Q R S T U V W X Y Z".split(" ");
	
	public CountingMapData(int size){
		if(size < 0) size = 0;
		this.size = size;
	}
	
	private static class Entry implements Map.Entry<Integer, String>{
		private int index;

		public Entry(int index) {
			super();
			this.index = index;
		}

		@Override
		public boolean equals(Object obj) {
			return Integer.valueOf(index).equals(obj);
		}

		@Override
		public int hashCode() {
			return Integer.valueOf(index).hashCode();
		}

		@Override
		public Integer getKey() {
			return index;
		}

		@Override
		public String getValue() {
			return chars[index % chars.length] +
					Integer.toString(index / chars.length);
		}

		@Override
		public String setValue(String value) {
			throw new UnsupportedOperationException();
		}	
	}
	
	@Override
	public Set<java.util.Map.Entry<Integer, String>> entrySet() {
		// LinkedHashSet retains initialization order
		Set<Map.Entry<Integer, String>> entries = 
				new LinkedHashSet<Map.Entry<Integer, String>>();
		for(int i = 0; i < size; i++)
			entries.add(new Entry(i));
		return entries;
	}

	public static void main(String[] args){
		Print.print(new CountingMapData(60));
	}
}


