/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */
package net.sourceforge.mysources.thinkinjava4.utils;

import java.util.LinkedHashMap;

/**
 * A Map filled with data using a generator object.
 *
 * @anthor Lucky Yang<1290308475@qq.com>
 * @Date 2012-12-29
 * @since 1.1.2
 */
public class MapData<K, V> extends LinkedHashMap<K, V> {
	/**
	 * 
	 */
	private static final long serialVersionUID = 5104122760121681166L;
	
	// A single Pare Generator
	public MapData(Generator<Pair<K, V>> gen, int quantity){
		for(int i = 0; i < quantity; i++){
			Pair<K, V> p = gen.next();
			put(p.key, p.value);
		}
	}
	
	// Two separate Generators
	public MapData(Generator<K> genK, Generator <V> genV,
			int quantity){
		for(int i = 0; i < quantity; i++){
			put(genK.next(), genV.next());
		}
	}
	
	// A key Generator and a single value
	public MapData(Generator<K> genK, V value, int quantity){
		for(int i = 0; i < quantity; i++){
			put(genK.next(), value);
		}
	}
	
	// An Iterable and a value Generator
	public MapData(Iterable<K> genK, Generator<V> genV){
		for(K key : genK){
			put(key, genV.next());
		}
	}
	
	// An Iterable and a single value
	public MapData(Iterable<K> genK, V value){
		for(K key : genK){
			put(key, value);
		}
	}
	
	// Generic convenience methods
	public static <K, V> MapData<K, V> map(Generator<Pair<K, V>> gen, int quantity){
		return new MapData<K, V>(gen, quantity);
	}
	public static <K, V> MapData<K, V> map(Generator<K> genK, Generator<V> genV, int quantity){
		return new MapData<K, V>(genK, genV, quantity);
	}	
	public static <K, V> MapData<K, V> map(Generator<K> genK, V value, int quantity){
		return new MapData<K, V>(genK, value, quantity);
	}	
	public static <K, V> MapData<K, V> map(Iterable<K> genK, Generator<V> genV){
		return new MapData<K, V>(genK, genV);
	}
	public static <K, V> MapData<K, V> map(Iterable<K> genK, V value){
		return new MapData<K, V>(genK, value);
	}	
}  


