/**
   Copyright (C) 2012 . see<http://mysources.sourceforge.net />

   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses />.
 */

package net.sourceforge.mysources.thinkinjava4.utils;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.TreeSet;

/**
 * 读取文本文件
 *
 * @author Lucky Yang
 * @since 1.0.0
 */
public class TextFile extends ArrayList<String> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2762285181346102154L;
	
	// Read a file as a single string
	public static String read(String fileName){
		StringBuilder sb = new StringBuilder();
		try{
			BufferedReader in = new BufferedReader(
					new FileReader(new File(fileName).getAbsoluteFile()));
			
			try {
				String s;
				while((s = in.readLine()) != null){
					sb.append(s);
					sb.append("\n");
				}
			}finally{
				in.close();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
		
		return sb.toString();
	}
	
	// Write a single file in one method call
	public static void write(String fileName, String text){
		try{
			PrintWriter out = new PrintWriter(
					new File(fileName).getAbsoluteFile());
			try{
				out.print(text);
			}finally{
				out.close();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}
	}
	
	// Read a file, split by any regular expression:
	public TextFile(String fileName, String splitter){
		super(Arrays.asList(read(fileName).split(splitter)));
		
		// Regular express split() often leaves an empty
		// String at the first position
		if(get(0).equals("")) remove(0);
	}
	
	// Normally read by lines:
	public TextFile(String fileName){
		this(fileName, "\n");
	}
	
	public void write(String fileName){
		try{
			PrintWriter out = new PrintWriter(
					new File(fileName).getAbsoluteFile());
			try{
				for(String item : this)
					out.println(item);
			}finally{
				out.close();
			}
		} catch (IOException e) {
			throw new RuntimeException(e);
		}		
	}
	
	public static void main(String[] args){
		String file = read("Copyright.txt");
		write("Copyright1.txt", file);
		
		TextFile text = new TextFile("Copyright.txt");
		text.write("Copyright2.txt");
		
		// Break into unique sorted list of word
		TreeSet<String> words = new TreeSet<String>(
				new TextFile("Copyright.txt" , "\\W+"));
		// Display the capitalized words
		Print.print(words.headSet("a"));
	}
}
